@@ -2,7 +2,7 @@
 #
 #	XS.pm : perl function definition for Term::ReadLine::Gnu
 #
-#	$Id: XS.pm 494 2015-09-06 05:45:53Z hayashi $
+#	$Id: XS.pm 501 2016-02-28 09:19:56Z hayashi $
 #
 #	Copyright (c) 1999 Hiroo Hayashi.  All rights reserved.
 #
@@ -16,11 +16,11 @@ use strict;
 use warnings;
 use AutoLoader 'AUTOLOAD';
 
-use vars qw($VERSION);
-$VERSION='1.27';	# added for CPAN
+our $VERSION;
+$VERSION='1.29';	# added for CPAN
 
 # make aliases
-use vars qw(%Attribs);
+our %Attribs;
 *Attribs = \%Term::ReadLine::Gnu::Attribs;
 
 use vars qw(*read_history);
@@ -361,8 +361,7 @@ sub get_history_event ( $$;$ ) {
 # Readline Library already uses, instead of Term::Caps.pm.
 
 # Some terminals do not support 'ue' (underline end).
-use vars qw(%term_no_ue);
-%term_no_ue = ( kterm => 1 );
+our %term_no_ue = ( kterm => 1 );
 
 sub ornaments {
     return $rl_term_set unless @_;
